//******************************************************************
//* PIC16F690 LCD DRIVER PROJECT
//* SERVO MAGAZINE
//* WRITTEN BY:  FRED EADY
//* 05/26/09
//* REVISIONS:
//*
//******************************************************************

#include	<pic.h>


// Settings for PIC16F88
__CONFIG (BORDIS & MCLRDIS & WDTDIS & INTIO
 & IESODIS & UNPROTECT & FCMDIS);
 
#define esc 0x1B

//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void init(void);
void interrupt TIMER_AUSART(void);
void bin2hex(char binchar);
void bin2decimal(char binchar);
char sendchar(char usart_data);
void init_EUSART(void);
//******************************************************************
//*	GLOBALS
//******************************************************************
unsigned int tmsecs1,imsecs1;
char tsecs1,isecs1;
char high_char,low_char;
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_RXint	  	RCIE = 0
#define		enable_RXint	  	RCIE = 1
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0

#define mdelay1(msecdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								tmsecs1 = 0;				\
								TMR1H = 0xF8;				\
								TMR1L = 0x4A;				\
								TIMER1ON;					\
								while(tmsecs1 < msecdelay);
								
#define	sdelay1(secdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tsecs1 = 0;					\
								TMR1H = 0xF8;				\
								TMR1L = 0x4A;				\
								TIMER1ON;					\
								while(tsecs1 < secdelay);
//******************************************************************
//*	  I/O DEFINITIONS
//******************************************************************
#define	act_led		RA4
//******************************************************************
//*	USART BUFFER DEFINITIONS
//******************************************************************
//1,2,4,8,16,32,64,128 or 256 bytes 
#define USART_RX_BUFFER_SIZE  16     
#define USART_RX_BUFFER_MASK ( USART_RX_BUFFER_SIZE - 1 )
#define USART_TX_BUFFER_SIZE  16      
#define USART_TX_BUFFER_MASK ( USART_TX_BUFFER_SIZE - 1 )
unsigned char USART_RxBuf[USART_RX_BUFFER_SIZE],USART_TxBuf[USART_TX_BUFFER_SIZE];
unsigned char USART_TxHead,USART_TxTail,USART_RxHead,USART_RxTail;

void putch(char c)
{
   while(!TRMT) 	// TRMT is set when TSR is empty
      continue;

  TXREG = c;		// load the register
}
//******************************************************************
//*	INTERRUPT HANDLER ROUTINE
//******************************************************************
void interrupt TIMER_AUSART(void)
{
	  char usart_lcddata,tmphead,tmptail;

	if((TMR1IF && TMR1IE))
	{
		TMR1IF = 0;
		TMR1H = 0xF8;	
		TMR1L = 0x4A;
		//act_led ^= 1;	//comment out for 1 second blinks
		++tmsecs1;
		if(++imsecs1 == 1000)
		{
			imsecs1 = 0;
			++tsecs1;
			act_led ^= 1; //comment out for 1 mS blinks
		}
	}

 
  if(TRMT)
  {
                                // check if all usart_lcddata is transmitted  
   if ( USART_TxHead != USART_TxTail )
   {
                                // calculate buffer index  
     tmptail = ( USART_TxTail + 1 ) & USART_TX_BUFFER_MASK;
     USART_TxTail = tmptail;      // store new index  

    TXREG = USART_TxBuf[tmptail];  // start transmitiON  
   }
   else
   {
     TXIE = 0;         // disable TX interrupt  
   }
  }

} 
//******************************************************************
//*	Init USART Function
//******************************************************************
void init_EUSART(void)
{
  SPBRG = 25;			//19200bps@8MHz
  TXSTA = 0x04;			//high speed baud rate-BRGH = 1	
  RCSTA = 0x80;			//enable serial port and serial port pins
  
  USART_TxTail = 0x00;	//flush transmit buffer 
  USART_TxHead = 0x00;
  RCIE = 0;				//disable receive interrupt
  CREN = 0;				//disable AUSART receiver
  TXIE = 0;				//disable USART1 transmit interrupt
  TXEN = 1;				//transmitter enabled
  RCIF = 0;				//clear receive interrupt flag
}

char sendchar(char usart_data)
{
  unsigned char tmphead;
                                // calculate buffer index 
  tmphead = ( USART_TxHead + 1 ) & USART_TX_BUFFER_MASK;
                                // wait for free space in buffer 
  while ( tmphead == USART_TxTail );
                                // store usart_data in buffer 
  //USART_TxBuf[tmphead] = (unsigned char)usart_data;
  USART_TxBuf[tmphead] = usart_data;
  USART_TxHead = tmphead;        // store new index 

  TXIE = 1;            // enable TX interrupt 

  return usart_data;
}
//******************************************************************
//*	INITIALIZE
//******************************************************************
void init(void)
{
	char init8;
   	TRISA = 0b11000000;		
   	TRISB = 0b01111100;	
   	TRISC = 0b00000000;	
//******************************************************************
//*	CONFIGURE EEPROM READ/WRITE
//******************************************************************
	EECON1 = 0b00000000;
//******************************************************************
//*	CONFIGURE A2D AND COMPARATORS
//******************************************************************
	ADCON0 = 0b00000000;
	ANSEL = 0b00000000;
	ANSELH = 0b00000000;
	C1ON = 0;
	C2ON = 0;			//comparators off
//******************************************************************
//*	INITIALIZE EUSART
//******************************************************************
	init_EUSART();
//******************************************************************
//*	CONFIGURE AND START TIMER1
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER1OFF;
	TMR1H = 0xF8;
	TMR1L = 0x4A;
	TIMER1ON;
//******************************************************************
//*	ENABLE INTERRUPTS
//******************************************************************
	enable_TMR1int;
    enable_GLOBALint;	
}  
//******************************************************************
//*	Converts Binary to Displayable Hex Characters
//*   ie.. 0x00 in gives 0x30 and 0x30 out
//******************************************************************
void bin2hex(char binchar)
{
   char high_nibble_out,low_nibble_out;

   high_nibble_out = (binchar & 0xF0) / 16;
   if(high_nibble_out > 0x09)
      high_char = high_nibble_out + 0x37;
   else
      high_char = high_nibble_out + 0x30;

   low_nibble_out = (binchar & 0x0F);
   if(low_nibble_out > 0x09)
      low_char = low_nibble_out + 0x37;
   else
      low_char = low_nibble_out + 0x30;
}
//******************************************************************
//*	Converts Binary to Displayable Hex Characters
//*   ie.. 0x00 in gives 0x30 and 0x30 out
//******************************************************************
void bin2decimal(char binchar)
{
	high_char = 0x30;
	low_char = 0x30;
	if(binchar >= 10)
	{
   		do{
			binchar -= 10;
			++high_char;
		}while(binchar >= 10);
	}
	low_char = low_char + binchar;
}
void main(void)
{
	char i;
	
	OSCCON = 0b01110111;	//8MHz internal clock
	init();
	i = 0;
	do{
		sdelay1(5);
		PORTC = i++;
		sendchar(PORTC);
	}while(1);	

}
