//******************************************************************
//* PIC88 LCD DRIVER PROJECT
//* SERVO MAGAZINE
//* WRITTEN BY:  FRED EADY
//* 05/26/09
//* REVISIONS:
//*
//******************************************************************

#include	<pic.h>


// Settings for PIC16F88
__CONFIG (DEBUGEN & LVPDIS & BORDIS & MCLREN & WDTDIS & INTIO
 & IESODIS & UNPROTECT & FCMDIS);
 
#define esc 0x1B

//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void init(void);
void interrupt TIMER_AUSART(void);
//void lcd_write( char mode, char nibble_out, char byte_out ); 
void lcd_send_byte(char address, char n);
void lcd_gotoxy( char x, char y); 
void lcd_init(void); 
void bin2hex(char binchar);
void bin2decimal(char binchar);
char recvchar(void);
char CharInQueue(void);
void init_AUSART(void);
//******************************************************************
//    MACROS AND PORT/PIN DEFINITIONS
//******************************************************************
#define bitset(var, bitno) ((var) |= 1 << (bitno))
#define bitclr(var, bitno) ((var) &= ~(1 << (bitno)))
#define make8(var,offset)	((unsigned int)var >> (offset * 8)) & 0x00FF
#define	make16(varhigh,varlow)	(((unsigned int)varhigh & 0xFF)* 0x100) + ((unsigned int)varlow & 0x00FF)
#define make32(var1,var2,var3,var4) \
		((unsigned long)var1<<24)+((unsigned long)var2<<16)+ \
		((unsigned long)var3<<8)+((unsigned long)var4)		
#define make32i(var1,var2) ((unsigned long)var1<<16)+((unsigned long)var2)
//******************************************************************
//*	GLOBALS
//******************************************************************
char bytein,binmask;
unsigned int tmsecs1,imsecs1;
char tsecs1,isecs1;
char high_char,low_char;
//******************************************************************
//*	CANNED LCD MESSAGES
//******************************************************************
const char LCD_INIT_STRING[5] = {0x28,0x08,0x01,0x06,0x0C};
//                        "12345678901234567890";
const char lcdmsg_banner1[]	= "   SERVO MAGAZINE   ";
const char lcdmsg_banner2[]	= "--------------------";
const char lcdmsg_banner3[]	= "  DOING BIG THINGS  ";
const char lcdmsg_banner4[]	= " WITH LITTLE MICROS ";
const char lcdmsg_portc[]		= "  PORTC = ";
const char lcdmsg_spaces[]  = "                    ";
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_RXint	  	RCIE = 0
#define		enable_RXint	  	RCIE = 1
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0

#define mdelay1(msecdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								tmsecs1 = 0;				\
								TMR1H = 0xF8;				\
								TMR1L = 0x4A;				\
								TIMER1ON;					\
								while(tmsecs1 < msecdelay);
								
#define	sdelay1(secdelay)		TIMER1OFF;					\
								TMR1IF = 0;					\
								imsecs1 = 0;				\
								tsecs1 = 0;					\
								TMR1H = 0xF8;				\
								TMR1L = 0x4A;				\
								TIMER1ON;					\
								while(tsecs1 < secdelay);
//******************************************************************
//*	  LCD I/O DEFINITIONS
//******************************************************************
#define databus	   	PORTA		//4-bit lcddata bus
#define lcdcntrl   	PORTB		//E and RS
#define RS			0b00000001 	//RB0
#define E 	   		0b00000010	//RB1
//#define RW  		GND 
//#define clrRW		lcdcntrl &= ~RW
//#define setRW		lcdcntrl |= RW
#define clrRS		lcdcntrl &= ~RS
#define setRS		lcdcntrl |= RS
#define clrE		lcdcntrl &= ~E
#define setE    	lcdcntrl |= E


#define	lcdcmd		0	//LCD command follows
#define lcddata		1	//LCD lcddata follows
#define hi			1	//send high nibble_out
#define low			0	//send low nibble_out
#define hilo		2	//send byte

#define lcdcol		20

#define lcdcls	   	lcd_send_byte(lcdcmd,0x01)
#define	line1	   	lcd_gotoxy(1,1)
#define	line2	   	lcd_gotoxy(2,1)
#define	line3	   	lcd_gotoxy(3,1)
#define	line4	   	lcd_gotoxy(4,1)
//******************************************************************
//*	  I/O DEFINITIONS
//******************************************************************
#define	act_led		RA4
//******************************************************************
//*	USART BUFFER DEFINITIONS
//******************************************************************
//1,2,4,8,16,32,64,128 or 256 bytes 
#define USART_RX_BUFFER_SIZE  16     
#define USART_RX_BUFFER_MASK ( USART_RX_BUFFER_SIZE - 1 )
unsigned char USART_RxBuf[USART_RX_BUFFER_SIZE];
unsigned char USART_RxHead,USART_RxTail;
//******************************************************************
//*	INTERRUPT HANDLER ROUTINE
//******************************************************************
void interrupt TIMER_AUSART(void)
{
	  char usart_lcddata,tmphead,tmptail;

	if((TMR1IF && TMR1IE))
	{
		TMR1IF = 0;
		TMR1H = 0xF8;	
		TMR1L = 0x4A;
		//act_led ^= 1;	//comment out for 1 second blinks
		++tmsecs1;
		if(++imsecs1 == 1000)
		{
			imsecs1 = 0;
			++tsecs1;
			act_led ^= 1; //comment out for 1 mS blinks
		}
	}

  if(RCIF && RCIE)
 {
   usart_lcddata = RCREG;                   // read the received usart_lcddata 
                                // calculate buffer index 
   tmphead = ( USART_RxHead + 1 ) & USART_RX_BUFFER_MASK;
   USART_RxHead = tmphead;        // store new index 

   if ( tmphead == USART_RxTail )
   {
     // ERROR! Receive buffer overflow 
     CREN = 0;
     CREN = 1;
   }
                                
   USART_RxBuf[tmphead] = usart_lcddata;   // store received usart_lcddata in buffer 
  }

} 
//******************************************************************
//*	Init USART Function
//******************************************************************
void init_AUSART(void)
{
  SPBRG = 25;			//19200bps@8MHz
  TXSTA = 0x04;			//high speed baud rate-BRGH = 1	
  RCSTA = 0x80;			//enable serial port and serial port pins
  
  USART_RxTail = 0x00;	//flush receive buffer 
  USART_RxHead = 0x00;
  RCIE = 1;			//enable receive interrupt
  CREN = 1;			//enable AUSART receiver
  TXIE = 0;			//disable USART1 transmit interrupt
  TXEN = 0;			//transmitter disabled
  RCIF = 0;
}

char recvchar(void)
{
  unsigned char tmptail;
                                // wait for incomming usart_lcddata  
  while ( USART_RxHead == USART_RxTail );
                                // calculate buffer index  
  tmptail = ( USART_RxTail + 1 ) & USART_RX_BUFFER_MASK;
  USART_RxTail = tmptail;        // store new index  

  return USART_RxBuf[tmptail];   // return usart_lcddata  
}
char CharInQueue(void)
{
  return(USART_RxHead != USART_RxTail);
}
//******************************************************************
//*	SEND NIBBLE TO LCD
//******************************************************************
void lcd_send_nibble( char n ) 
{
	databus = n >> 4;
	mdelay1(1);
	setE;
	mdelay1(1);
	clrE;
}
//******************************************************************
//*	SEND BYTE TO LCD
//******************************************************************
void lcd_send_byte( char address, char n ) 
{
      clrE;
      
	  switch (address)
	  {
	  		 case lcdcmd:
			 	  clrRS;
				  break;
			 case lcddata:
			 	  setRS;
				  break;
      }		
	  		mdelay1(1);
		    
      lcd_send_nibble (n);
      lcd_send_nibble (n << 4);
	  		mdelay1(1);

}

//******************************************************************
//*	INITIALIZE THE LCD
//******************************************************************
void lcd_init(void) 
{
    char j8;
    clrRS;
    clrE;
	mdelay1(16);
	lcd_send_nibble(0x30);
	mdelay1(6);
	lcd_send_nibble(0x30);
	mdelay1(1);
	lcd_send_nibble(0x30);
	mdelay1(2);
    lcd_send_nibble(0x20);
	for(j8=0;j8<sizeof(LCD_INIT_STRING);++j8)
	{
	mdelay1(2);
       lcd_send_byte(lcdcmd,LCD_INIT_STRING[j8]);
	}  
} 
void lcd_gotoxy( char x, char y) 
{
// where x = lcd row (1,2,3,4) and y = column (1 thru 20)
   char address;
   switch (x)
   {
   		  case 1:
		  	   address = 0;
			   break;
		  case 2:
		  	   address = 0x40;
			   break;
		  case 3:
		  	   address = 0x14;
			   break;
		  case 4:
		  	   address = 0x54;
			   break;
		  default:
		  	   address = 0;
   }   
   address += (y-1);
   lcd_send_byte(lcdcmd,0x80|address);
}
//******************************************************************
//*	INITIALIZE
//******************************************************************
void init(void)
{
	char init8;
   	TRISA = 0b11100000;		
   	TRISB = 0b11111100;		
//******************************************************************
//*	CONFIGURE EEPROM READ/WRITE
//******************************************************************
	EECON1 = 0b00000000;
//******************************************************************
//*	CONFIGURE A2D AND COMPARATORS
//******************************************************************
	ANSEL = 0b00000000;		//PORTA all digital
	CMCON = 0x07;			//comparators off
//******************************************************************
//*	INITIALIZE EUSART
//******************************************************************
	init_AUSART();
//******************************************************************
//*	CONFIGURE AND START TIMER1
//* SET TO OVERFLOW EVERY 1mS
//******************************************************************
	TIMER1OFF;
	TMR1H = 0xF8;
	TMR1L = 0x4A;
	TIMER1ON; 
//******************************************************************
//*	ENABLE INTERRUPTS
//******************************************************************
	enable_TMR1int;
    enable_GLOBALint;	
//******************************************************************
//*	INITIALIZE GLOBAL VARIABLES
//******************************************************************
//******************************************************************
//*	INITIALIZE LCD
//******************************************************************
		lcd_init();
		line1;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,lcdmsg_banner1[init8]);
			} 
		line2;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,lcdmsg_banner2[init8]);
			} 
		line3;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,lcdmsg_banner3[init8]);
			} 
		line4;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,lcdmsg_banner4[init8]);
			}
		sdelay1(30);
		line1;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,lcdmsg_banner1[init8]);
			} 
		line2;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,lcdmsg_banner2[init8]);
			} 
		line3;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,0x20);
			} 
		line4;
			for(init8=0;init8<lcdcol;++init8)
			{
				lcd_send_byte(lcddata,0x20);
			}

}  
//******************************************************************
//*	Converts Binary to Displayable Hex Characters
//*   ie.. 0x00 in gives 0x30 and 0x30 out
//******************************************************************
void bin2hex(char binchar)
{
   char high_nibble_out,low_nibble_out;

   high_nibble_out = (binchar & 0xF0) / 16;
   if(high_nibble_out > 0x09)
      high_char = high_nibble_out + 0x37;
   else
      high_char = high_nibble_out + 0x30;

   low_nibble_out = (binchar & 0x0F);
   if(low_nibble_out > 0x09)
      low_char = low_nibble_out + 0x37;
   else
      low_char = low_nibble_out + 0x30;
}
//******************************************************************
//*	Converts Binary to Displayable Hex Characters
//*   ie.. 0x00 in gives 0x30 and 0x30 out
//******************************************************************
void bin2decimal(char binchar)
{
	high_char = 0x30;
	low_char = 0x30;
	if(binchar >= 10)
	{
   		do{
			binchar -= 10;
			++high_char;
		}while(binchar >= 10);
	}
	low_char = low_char + binchar;
}
void main(void)
{
	char i,mark;
	
	OSCCON = 0b01111110;	//8MHz internal clock
	init();
	
	do{
		while(!(CharInQueue()));
		bytein = recvchar();

		binmask = 0b10000000;
		line3;
		for(i=0;i<sizeof(lcdmsg_portc)-1;++i)
		{
			lcd_send_byte(lcddata,lcdmsg_portc[i]);
		}	
		for(i=0;i<8;++i)
		{
			if((binmask & bytein) == 0)
			{
				lcd_send_byte(lcddata,'0');
			}
			else
			{
				lcd_send_byte(lcddata,'1');
			}
			binmask = binmask >> 1;
		}

	}while(1);	

}
